//+------------------------------------------------------------------+
//|                                      Exp_CandleStop_Trailing.mq5 |
//|                             Copyright  2012,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
#property version   "1.00"
//+----------------------------------------------+
//|                      |
//+----------------------------------------------+
input uint UpTrailPeriods=5; //                  
input uint DnTrailPeriods=5; //   
input uint Deviation=10;  //
//+----------------------------------------------+

//----      
int InpInd_Handle;
//----      
int min_rates_total;
//+------------------------------------------------------------------+
//|   CChart                           |
//+------------------------------------------------------------------+
#include <Charts\Chart.mqh>
//----     CChart
CChart cchart;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//----    CandleStop
   InpInd_Handle=iCustom(Symbol(),PERIOD_CURRENT,"CandleStop",UpTrailPeriods,DnTrailPeriods,0);
   if(InpInd_Handle==INVALID_HANDLE) Print("      CandleStop");

//---     
   ResetLastError();

//---   cchart    (ID=0) ,    
   cchart.Attach(0);

//----   ATR_Trailing    
   if(!cchart.IndicatorAdd(0,InpInd_Handle)) Print("     CandleStop  ");

//----     
   min_rates_total=int(MathMax(UpTrailPeriods,DnTrailPeriods))+1;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//----
//----
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//----       
   if(BarsCalculated(InpInd_Handle)<min_rates_total) return;

//----   
   double DnValue[2],UpValue[2],NewStop;

//----   
   static bool Recount=true,BuySignal,SellSignal;
   static CIsNewBar NB;

//----      
   if(!PositionSelect(Symbol())) return;

//----    
   ENUM_POSITION_TYPE PosType=ENUM_POSITION_TYPE(PositionGetInteger(POSITION_TYPE));
   double LastStop=PositionGetDouble(POSITION_SL);

   if(!LastStop)
     {
      //----    
      BuySignal=false;
      SellSignal=false;

      //----      
      if(CopyBuffer(InpInd_Handle,1,0,2,UpValue)<=0) return;
      if(CopyBuffer(InpInd_Handle,2,0,2,DnValue)<=0) return;

      switch(PosType)
        {
         case POSITION_TYPE_SELL:
           {
            NewStop=MathMax(UpValue[0],UpValue[1]);
            double Bid=SymbolInfoDouble(Symbol(),SYMBOL_BID);
            if(!Bid || NewStop<=Bid) return;
            SellSignal=true;
            break;
           }

         case POSITION_TYPE_BUY:
           {
            NewStop=MathMin(DnValue[0],DnValue[1]);
            double Ask=SymbolInfoDouble(Symbol(),SYMBOL_ASK);
            if(!Ask || NewStop>=Ask) return;
            BuySignal=true;
            break;
           }
         default: return;
        }
     }
   else if(NB.IsNewBar(Symbol(),PERIOD_CURRENT) || Recount) //     
     {
      //----      
      Recount=false;

      //----    
      BuySignal=false;
      SellSignal=false;

      //----      
      if(CopyBuffer(InpInd_Handle,1,0,2,UpValue)<=0) {Recount=true; return;}
      if(CopyBuffer(InpInd_Handle,2,0,2,DnValue)<=0) {Recount=true; return;}

      switch(PosType)
        {
         case POSITION_TYPE_SELL:
           {
            NewStop=UpValue[0];
            if(NewStop>=LastStop) return;
            SellSignal=true;
            break;
           }

         case POSITION_TYPE_BUY:
           {
            NewStop=DnValue[0];
            if(NewStop<=LastStop) return;
            BuySignal=true;
            break;
           }
         default: return;
        }
     }
//+----------------------------------------------+
//|                              |
//+----------------------------------------------+
//----  
   dBuyPositionModify(BuySignal,Symbol(),Deviation,NewStop,0.0);

//----   
   dSellPositionModify(SellSignal,Symbol(),Deviation,NewStop,0.0);
//----
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
bool dBuyPositionModify
(
 bool &Modify_Signal,        //   
 const string symbol,        //   
 uint deviation,             // 
 double StopLoss,            //      
 double Takeprofit           //      
 )
//dBuyPositionModify(Modify_Signal,symbol,deviation,StopLoss,Takeprofit);
  {
//----
   if(!Modify_Signal) return(true);

   ENUM_POSITION_TYPE PosType=POSITION_TYPE_BUY;

//----      
   if(!PositionSelect(symbol)) return(true);
   if(PositionGetInteger(POSITION_TYPE)!=PosType) return(true);

//----        
   MqlTradeRequest request;
   MqlTradeResult result;

//----       
   MqlTradeCheckResult check;

//----  
   ZeroMemory(request);
   ZeroMemory(result);
   ZeroMemory(check);
//----
   int digit=int(SymbolInfoInteger(symbol,SYMBOL_DIGITS));
   double point=SymbolInfoDouble(symbol,SYMBOL_POINT);
   double Ask=SymbolInfoDouble(symbol,SYMBOL_ASK);
   if(!digit || !point || !Ask) return(true);

//----     MqlTradeRequest   BUY 
   request.type   = ORDER_TYPE_BUY;
   request.price  = Ask;
   request.action = TRADE_ACTION_SLTP;
   request.symbol = symbol;

//----        
   if(StopLoss)
     {
      int nStopLoss=int((Ask-StopLoss)/point);
      if(nStopLoss<0) return(false);
      if(!StopCorrect(symbol,nStopLoss))return(false);
      double dStopLoss=nStopLoss*point;
      request.sl=NormalizeDouble(request.price-dStopLoss,digit);
      if(request.sl<PositionGetDouble(POSITION_SL)) request.sl=PositionGetDouble(POSITION_SL);
     }
   else request.sl=PositionGetDouble(POSITION_SL);

//----       
   if(Takeprofit)
     {
      int nTakeprofit=int((Takeprofit-Ask)/point);
      if(nTakeprofit<0) return(false);
      if(!StopCorrect(symbol,nTakeprofit))return(false);
      double dTakeprofit=nTakeprofit*point;
      request.tp=NormalizeDouble(request.price+dTakeprofit,digit);
      if(request.tp<PositionGetDouble(POSITION_TP)) request.tp=PositionGetDouble(POSITION_TP);
     }
   else request.tp=PositionGetDouble(POSITION_TP);

//----   
   if(request.tp==PositionGetDouble(POSITION_TP) && request.sl==PositionGetDouble(POSITION_SL)) return(true);
   request.deviation=deviation;
   request.type_filling=ORDER_FILLING_FOK;

//----     
   if(!OrderCheck(request,check))
     {
      Print(__FUNCTION__,"():      !");
      Print(__FUNCTION__,"(): OrderCheck(): ",ResultRetcodeDescription(check.retcode));
      return(false);
     }

   string comment="";
   StringConcatenate(comment,"<<< ============ ",__FUNCTION__,"():  Buy   ",symbol," ============ >>>");
   Print(comment);

//----  BUY       
   if(!OrderSend(request,result) || result.retcode!=TRADE_RETCODE_DONE)
     {
      Print(__FUNCTION__,"():   !");
      Print(__FUNCTION__,"(): OrderSend(): ",ResultRetcodeDescription(result.retcode));
      return(false);
     }
   else
   if(result.retcode==TRADE_RETCODE_DONE)
     {
      Modify_Signal=false;
      comment="";
      StringConcatenate(comment,"<<< ============ ",__FUNCTION__,"(): Buy   ",symbol,"  ============ >>>");
      Print(comment);
      PlaySound("ok.wav");
     }
   else
     {
      Print(__FUNCTION__,"():   !");
      Print(__FUNCTION__,"(): OrderSend(): ",ResultRetcodeDescription(result.retcode));
     }
//----
   return(true);
  }
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
bool dSellPositionModify
(
 bool &Modify_Signal,        //   
 const string symbol,        //   
 uint deviation,             // 
 double StopLoss,            //      
 double Takeprofit           //      
 )
//dSellPositionModify(Modify_Signal,symbol,deviation,StopLoss,Takeprofit);
  {
//----
   if(!Modify_Signal) return(true);

   ENUM_POSITION_TYPE PosType=POSITION_TYPE_SELL;

//----      
   if(!PositionSelect(symbol)) return(true);
   if(PositionGetInteger(POSITION_TYPE)!=PosType) return(true);

//----        
   MqlTradeRequest request;
   MqlTradeResult result;

//----       
   MqlTradeCheckResult check;

//----  
   ZeroMemory(request);
   ZeroMemory(result);
   ZeroMemory(check);
//----
   int digit=int(SymbolInfoInteger(symbol,SYMBOL_DIGITS));
   double point=SymbolInfoDouble(symbol,SYMBOL_POINT);
   double Ask=SymbolInfoDouble(symbol,SYMBOL_ASK);
   if(!digit || !point || !Ask) return(true);

//----     MqlTradeRequest   BUY 
   request.type   = ORDER_TYPE_SELL;
   request.price  = Ask;
   request.action = TRADE_ACTION_SLTP;
   request.symbol = symbol;

//----        
   if(StopLoss!=0)
     {
      int nStopLoss=int((StopLoss-Ask)/point);
      if(nStopLoss<0) return(false);
      if(!StopCorrect(symbol,nStopLoss))return(false);
      double dStopLoss=nStopLoss*point;
      request.sl=NormalizeDouble(request.price+dStopLoss,digit);
      double laststop=PositionGetDouble(POSITION_SL);
      if(request.sl>laststop && laststop) request.sl=PositionGetDouble(POSITION_SL);
     }
   else request.sl=PositionGetDouble(POSITION_SL);

//----       
   if(Takeprofit!=0)
     {
      int nTakeprofit=int((Ask-Takeprofit)/point);
      if(nTakeprofit<0) return(false);
      if(!StopCorrect(symbol,nTakeprofit))return(false);
      double dTakeprofit=nTakeprofit*point;
      request.tp=NormalizeDouble(request.price-dTakeprofit,digit);
      double lasttake=PositionGetDouble(POSITION_TP);
      if(request.tp>lasttake && lasttake) request.tp=PositionGetDouble(POSITION_TP);
     }
   else request.tp=PositionGetDouble(POSITION_TP);

//----   
   if(request.tp==PositionGetDouble(POSITION_TP) && request.sl==PositionGetDouble(POSITION_SL)) return(true);
   request.deviation=deviation;
   request.type_filling=ORDER_FILLING_FOK;

//----     
   if(!OrderCheck(request,check))
     {
      Print(__FUNCTION__,"():      !");
      Print(__FUNCTION__,"(): OrderCheck(): ",ResultRetcodeDescription(check.retcode));
      return(false);
     }

   string comment="";
   StringConcatenate(comment,"<<< ============ ",__FUNCTION__,"():  Sell   ",symbol," ============ >>>");
   Print(comment);

//----  SELL       
   if(!OrderSend(request,result) || result.retcode!=TRADE_RETCODE_DONE)
     {
      Print(__FUNCTION__,"():   !");
      Print(__FUNCTION__,"(): OrderSend(): ",ResultRetcodeDescription(result.retcode));
      return(false);
     }
   else
   if(result.retcode==TRADE_RETCODE_DONE)
     {
      Modify_Signal=false;
      comment="";
      StringConcatenate(comment,"<<< ============ ",__FUNCTION__,"(): Sell   ",symbol,"  ============ >>>");
      Print(comment);
      PlaySound("ok.wav");
     }
   else
     {
      Print(__FUNCTION__,"():   !");
      Print(__FUNCTION__,"(): OrderSend(): ",ResultRetcodeDescription(result.retcode));
     }
//----
   return(true);
  }
//+------------------------------------------------------------------+
//|            |
//+------------------------------------------------------------------+
bool StopCorrect(string symbol,int &Stop)
  {
//----
   int Extrem_Stop=int(SymbolInfoInteger(symbol,SYMBOL_TRADE_STOPS_LEVEL));
   if(!Extrem_Stop) return(false);
   if(Stop<Extrem_Stop) Stop=Extrem_Stop;
//----
   return(true);
  }
//+------------------------------------------------------------------+
//|                     |
//+------------------------------------------------------------------+  
class CIsNewBar
  {
   //----
public:
   //----      
   bool IsNewBar(string symbol,ENUM_TIMEFRAMES timeframe)
     {
      //----     
      datetime TNew=datetime(SeriesInfoInteger(symbol,timeframe,SERIES_LASTBAR_DATE));

      if(TNew!=m_TOld && TNew) //     
        {
         m_TOld=TNew;
         return(true); //   !
        }
      //----
      return(false); //    !
     };

   //----      
                     CIsNewBar(){m_TOld=-1;};

protected: datetime m_TOld;
   //---- 
  };
//+------------------------------------------------------------------+
//|             |
//+------------------------------------------------------------------+
string ResultRetcodeDescription(int retcode)
  {
   string str;
//----
   switch(retcode)
     {
      case TRADE_RETCODE_REQUOTE: str=""; break;
      case TRADE_RETCODE_REJECT: str=" "; break;
      case TRADE_RETCODE_CANCEL: str="  "; break;
      case TRADE_RETCODE_PLACED: str=" "; break;
      case TRADE_RETCODE_DONE: str=" "; break;
      case TRADE_RETCODE_DONE_PARTIAL: str="  "; break;
      case TRADE_RETCODE_ERROR: str="  "; break;
      case TRADE_RETCODE_TIMEOUT: str="    ";break;
      case TRADE_RETCODE_INVALID: str=" "; break;
      case TRADE_RETCODE_INVALID_VOLUME: str="   "; break;
      case TRADE_RETCODE_INVALID_PRICE: str="   "; break;
      case TRADE_RETCODE_INVALID_STOPS: str="   "; break;
      case TRADE_RETCODE_TRADE_DISABLED: str=" "; break;
      case TRADE_RETCODE_MARKET_CLOSED: str=" "; break;
      case TRADE_RETCODE_NO_MONEY: str="      "; break;
      case TRADE_RETCODE_PRICE_CHANGED: str=" "; break;
      case TRADE_RETCODE_PRICE_OFF: str="    "; break;
      case TRADE_RETCODE_INVALID_EXPIRATION: str="     "; break;
      case TRADE_RETCODE_ORDER_CHANGED: str="  "; break;
      case TRADE_RETCODE_TOO_MANY_REQUESTS: str="  "; break;
      case TRADE_RETCODE_NO_CHANGES: str="   "; break;
      case TRADE_RETCODE_SERVER_DISABLES_AT: str="  "; break;
      case TRADE_RETCODE_CLIENT_DISABLES_AT: str="   "; break;
      case TRADE_RETCODE_LOCKED: str="   "; break;
      case TRADE_RETCODE_FROZEN: str="   "; break;
      case TRADE_RETCODE_INVALID_FILL: str="       "; break;
      case TRADE_RETCODE_CONNECTION: str="    "; break;
      case TRADE_RETCODE_ONLY_REAL: str="     "; break;
      case TRADE_RETCODE_LIMIT_ORDERS: str="     "; break;
      case TRADE_RETCODE_LIMIT_VOLUME: str="         "; break;
      default: str=" ";
     }
//----
   return(str);
  }
//+------------------------------------------------------------------+
